#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
:copyright:
    Yangkang Chen (chenyk2016@gmail.com), 2022-Present 
:license:
    GNU General Public License, Version 3
    (http://www.gnu.org/copyleft/gpl.html)
"""

__version__ = "0.0.8"

from .fmm import eikonal
from .fmm import eikonal_surf
from .fmm import eikonal_rtp

from eikonalc import *















#!/usr/bin/env python
# -*- encoding: utf8 -*-
import glob
import inspect
import io
import os

from setuptools import find_packages
from setuptools import setup


long_description = """
Source code: https://github.com/chenyk1990/pyekfmm""".strip() 

def read(*names, **kwargs):
    return io.open(
        os.path.join(os.path.dirname(__file__), *names),
        encoding=kwargs.get("encoding", "utf8")).read()


from distutils.core import Extension
import numpy
eikonalc_module = Extension('eikonalc', sources=['pyekfmm/src/eikonal.c'], 
										include_dirs=[numpy.get_include()])

# from numpy.distutils.core import setup 
# from distutils.core import setup
setup(
    name="pyekfmm",
    version="0.0.8.2",
    license='GNU General Public License, Version 3 (GPLv3)',
    description="Fast Marching Method for Traveltime Calculation",
    long_description=long_description,
    author="pyekfmm developing team",
    author_email="chenyk2016@gmail.com",
    url="https://github.com/chenyk1990/pyekfmm",
    ext_modules=[eikonalc_module],
    packages=['pyekfmm'],
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "Operating System :: Unix",
        "Operating System :: POSIX",
        "Operating System :: Microsoft :: Windows",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: Implementation :: CPython",
        "Topic :: Scientific/Engineering",
        "Topic :: Scientific/Engineering :: Physics",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)"
    ],
    keywords=[
        "seismology", "exploration seismology", "array seismology", "denoising", "science", "signal-to-noise ratio", "damped rank reduction method"
    ],
    install_requires=[
        "numpy", "scipy", "matplotlib"
    ],
    extras_require={
        "docs": ["sphinx", "ipython", "runipy"]
    }
    
)



def eikonal(vel,xyz,ax=[0,0.01,101],ay=[0,0.01,101],az=[0,0.01,101],order=2,verb=1,angle=False):
	'''
	EIKONAL: Fast marching eikonal solver (3-D)
	
	INPUT
	vel: 1D numpy array (nx*ny*nz)
	xyz: 1D/2D numpy array (one event: 1x3 or multi-event: ne x 3)
	ax: axis x [ox,dx,nx]
	ay: axis y [oy,dy,ny]
	az: axis z [oz,dz,nz]
	order: accuracy order [1 or 2]
	
	OUTPUT
	times: traveltime in xyz respectively (1D numpy array)
		   (one event: nx*ny*nz or multi-event: nx*ny*nz*ne)
	
	EXAMPLE
	demos/test_first-fifth.py
	
	COPYRIGHT
	Yangkang Chen, 2022, The University of Texas at Austin
	
	MODIFICATIONS
	[1] By Yangkang Chen, Sep, 2022
	
	'''
	import numpy as np
	if xyz.size == 3:
		from eikonalc import eikonalc_oneshot
		x=xyz[0];y=xyz[1];z=xyz[2];
		if angle:
			
			from eikonalc import eikonalc_oneshot_angle
			n123=vel.size
			tmp=eikonalc_oneshot_angle(vel,x,y,z,ax[0],ay[0],az[0],ax[1],ay[1],az[1],ax[2],ay[2],az[2],order);
			times=tmp[0:n123]
			dips=tmp[n123:n123*2]
			azims=tmp[n123*2:]
		else:
			times=eikonalc_oneshot(vel,x,y,z,ax[0],ay[0],az[0],ax[1],ay[1],az[1],ax[2],ay[2],az[2],order);
	else:
		from eikonalc import eikonalc_multishots
		[ne,ndim]=xyz.shape;#ndim must be 3
		x=xyz[:,0];y=xyz[:,1];z=xyz[:,2];
		x=np.expand_dims(x,1);
		y=np.expand_dims(y,1);
		z=np.expand_dims(z,1);
		
		if angle:
			from eikonalc import eikonalc_multishots_angle
			n123=vel.size
			tmp=eikonalc_multishots_angle(vel,x,y,z,ax[0],ay[0],az[0],ax[1],ay[1],az[1],ax[2],ay[2],az[2],order,verb);
			times=tmp[0:n123*ne]
			dips=tmp[n123*ne:n123*ne*2]
			azims=tmp[n123*ne*2:]
			
		else:
			times=eikonalc_multishots(vel,x,y,z,ax[0],ay[0],az[0],ax[1],ay[1],az[1],ax[2],ay[2],az[2],order,verb);
	
	if angle:
		return times,dips,azims
	else:
		return times
	
	
def eikonal_surf(vel,xyz,ax=[0,0.01,101],ay=[0,0.01,101],az=[0,0.01,101],order=2,verb=1):
	'''
	EIKONAL_SURF: Fast marching eikonal solver (3-D) and recording the traveltimes on the surface.
	
	INPUT
	vel: 1D numpy array (nx*ny*nz)
	xyz: 1D/2D numpy array (one event: 1x3 or multi-event: ne x 3)
	ax: axis x [ox,dx,nx]
	ay: axis y [oy,dy,ny]
	az: axis z [oz,dz,nz]
	order: accuracy order [1 or 2]
	
	OUTPUT
	times: traveltime in xy of nshots (1D numpy array)
		   size of times is nx*ny*nshots
	
	EXAMPLE
	demos/test_fourth3d.py
	
	COPYRIGHT
	Yangkang Chen, 2022, The University of Texas at Austin
	
	MODIFICATIONS
	[1] By Yangkang Chen, Sep, 2022
	'''
	from eikonalc import eikonalc_surf
	[ne,ndim]=xyz.shape;#ndim must be 3
	x=xyz[:,0];y=xyz[:,1];z=xyz[:,2];
	times=eikonalc_surf(vel,x,y,z,ax[0],ay[0],az[0],ax[1],ay[1],az[1],ax[2],ay[2],az[2],order,verb);
		
	return times
	
def eikonal_rtp(vel,rtp,ar=[0,0.01,101],at=[0,1.8,101],ap=[0,3.6,101],order=2,verb=1):
	'''
	EIKONAL_RTP: Fast marching eikonal solver (3-D) in spherical coordinates
	
	INPUT
	vel: 1D numpy array (nx*ny*nz)
	rtp: 1D/2D numpy array (one event: 1x3 or multi-event: ne x 3)
	ar: axis r [or,dr,nr]
	at: axis theta [ot,dt,nt]
	ap: axis phi [op,dp,np]
	order: accuracy order [1 or 2]
	
	OUTPUT
	times: traveltime in xyz respectively (1D numpy array)
		   (one event: nx*ny*nz or multi-event: nx*ny*nz*ne)
	
	EXAMPLE
	demos/test_first-fifth.py
	
	COPYRIGHT
	Yangkang Chen, 2022, The University of Texas at Austin
	
	MODIFICATIONS
	[1] By Yangkang Chen, Sep, 2022
	
	'''
	import numpy as np
	if rtp.size == 3:
		from eikonalc import eikonalc_oneshot_rtp
		r=rtp[0];t=rtp[1];p=rtp[2];
		times=eikonalc_oneshot_rtp(vel,r,t,p,ar[0],at[0],ap[0],ar[1],at[1],ap[1],ar[2],at[2],ap[2],order);
	else:
		from eikonalc import eikonalc_multishots_rtp
		[ne,ndim]=rtp.shape;#ndim must be 3
		r=rtp[:,0];t=rtp[:,1];p=rtp[:,2];
		r=np.expand_dims(r,1);
		t=np.expand_dims(t,1);
		p=np.expand_dims(p,1);
		times=eikonalc_multishots_rtp(vel,r,t,p,ar[0],at[0],ap[0],ar[1],at[1],ap[1],ar[2],at[2],ap[2],order,verb);
		
	return times
	
	